@@ -155,7 +155,8 @@
      UNGRUCK_LEFT = 371,
      AT_OP = 372,
      AT_CTOR = 373,
-     AT_DTOR = 374
+     AT_DTOR = 374,
+     AT_IMPORT = 375
    };
 #endif
 /* Tokens.  */
@@ -276,6 +277,7 @@
 #define AT_OP 372
 #define AT_CTOR 373
 #define AT_DTOR 374
+#define AT_IMPORT 375
 
 
 
@@ -307,9 +309,10 @@ typedef union YYSTYPE
     a_Complex complex_exp;
     a_Polar polar_exp;
     a_Vec vec_exp; // ge: added 1.3.5.3
+    a_Import import; // 1.5.2.5 (ge) added
 }
 /* Line 1529 of yacc.c.  */
-#line 313 "chuck.tab.h"
+#line 316 "chuck.tab.h"
 	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
